==> copilot-24.3.5-sources.jar: com/vaadin/copilot/ProjectManager.java <==
package com.vaadin.copilot;

import org.apache.commons.io.FileUtils;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.NoSuchFileException;
import java.nio.file.Path;

public class ProjectManager {
    private final Path projectRoot;

    public ProjectManager(Path projectRoot) throws IOException {
        this.projectRoot = projectRoot.toRealPath();
    }

    public String readFile(String filename) throws IOException {
        File file = new File(filename);
        if (isFileInsideProject(file)) {
            return FileUtils.readFileToString(file, StandardCharsets.UTF_8);
        } else {
            throw new IllegalArgumentException("File " + filename
                    + " is not inside the project at " + projectRoot);
        }
    }

    boolean isFileInsideProject(File file) throws IOException {
        Path path = getRealPath(file);
        return (path.startsWith(projectRoot));
    }

    private Path getRealPath(File file) throws IOException {
        Path path = file.toPath();
        try {
            return path.toRealPath();
        } catch (NoSuchFileException e) {
            // As we allow creating new files, we check the directory instead
            return path.getParent().toRealPath().resolve(path.getFileName());
        }
    }

    public void writeFile(String filename, String content) throws IOException {
        File file = new File(filename);
        if (isFileInsideProject(file)) {
            FileUtils.write(new File(filename), content,
                    StandardCharsets.UTF_8);
        } else {
            throw new IllegalArgumentException("File " + filename
                    + " is not inside the project at " + projectRoot);
        }
    }

}

==> copilot-24.3.11-sources.jar: com/vaadin/copilot/ProjectManager.java <==
package com.vaadin.copilot;

import org.apache.commons.io.FileUtils;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.NoSuchFileException;
import java.nio.file.Path;

public class ProjectManager {
    private final Path projectRoot;

    public ProjectManager(Path projectRoot) throws IOException {
        this.projectRoot = projectRoot.toRealPath();
    }

    public String readFile(String filename) throws IOException {
        File file = new File(filename);
        if (isFileInsideProject(file)) {
            return FileUtils.readFileToString(file, StandardCharsets.UTF_8);
        } else {
            throw new IllegalArgumentException("File " + filename
                    + " is not inside the project at " + projectRoot);
        }
    }

    boolean isFileInsideProject(File file) throws IOException {
        Path path = getRealPath(file);
        return (path.startsWith(projectRoot));
    }

    private Path getRealPath(File file) throws IOException {
        Path path = file.toPath();
        try {
            return path.toRealPath();
        } catch (NoSuchFileException e) {
            // As we allow creating new files, we check the directory instead
            return path.getParent().toRealPath().resolve(path.getFileName());
        }
    }

    public void writeFile(String filename, String content) throws IOException {
        File file = new File(filename);
        if (isFileInsideProject(file)) {
            FileUtils.write(new File(filename), content,
                    StandardCharsets.UTF_8);
        } else {
            throw new IllegalArgumentException("File " + filename
                    + " is not inside the project at " + projectRoot);
        }
    }

    public String makeAbsolute(String projectRelativeFilename)
            throws IOException {
        Path resolved = projectRoot.resolve(projectRelativeFilename)
                .toRealPath();
        if (!resolved.startsWith(projectRoot)) {
            throw new IllegalArgumentException("File " + projectRelativeFilename
                    + " is not inside the project at " + projectRoot);
        }
        return resolved.toString();
    }

    public String makeRelative(String filename) throws IOException {
        Path absolutePath = new File(filename).toPath().toRealPath();
        // If file is not inside project root, throw
        if (!absolutePath.startsWith(projectRoot)) {
            throw new IllegalArgumentException("File " + filename
                    + " is not inside the project at " + projectRoot);
        }
        return projectRoot.relativize(absolutePath).toString();
    }
}
